!
! ml 15/11/99
!
! This program uses the parameters and data structures defined in
! edusoft_mod and make a call to all the routines included in
! edusoft_mod and emutils_mod. (except sizeListParam and readListParam).
 

PROGRAM check

  use emutils_mod
  use edusoft_mod
  use cal
  use dal
  use param
  use errorhandling

  type(edu_evt) :: eduevt
  type(edu_pattern) :: edupat(edu_npat)
  type(edu_out)     :: eduout

  integer, parameter :: dx = 100, dy = 100
  integer :: x0 =  10, y0 =  10 
  integer :: edumode = 3  ! Image mode
  integer :: ii, jj
  integer(kind=int16), dimension(0:1023)         :: offX
  integer(kind=int16), dimension(0:1023)         :: offY  
  integer(kind=int8),  dimension(:,:,:), pointer :: patterns
  integer(kind=int16)                            :: eduThreshold
  integer(kind=int16), dimension(dx,dy)          :: im

  integer(kind=int16), dimension(-1:1,-1:1,0:edu_npat-1) :: patabove
  type(sas_evt), dimension(1)  :: evt
  integer(kind=int32), dimension(-2:10,-2:10)        :: imagein
  integer :: npattern, jr

! For emutils 
  type(TableT)      :: ev_tab
  type(DataSetT)    :: ev_set
  character(len=80) :: filename = "../emodf/0001_183536AIV-_M1S00170IME.FIT"

  logical           :: imag, timi, redI, comp
  character(len=*), parameter :: taskname = "test_emsaplib"
  character(len=8), dimension(9), parameter :: keycompat = &
         (/"TELESCOP","INSTRUME","CCDID   ","CCDNODE ",&
           "WINDOWX0","WINDOWY0","WINDOWDX","WINDOWDY","FRMTIME "/)

  eduevt%x = 4

  write(errstr,*) "PARAMETER VALUES :"
  call message(AppMsg,Verbose)

  write(errstr,*) "edu_npat = ",edu_npat
  call message(AppMsg,Verbose)

  write(errstr,*) "edu_nsid = ",edu_nsid
  call message(AppMsg,Verbose)

  write(errstr,*) "es_nmax  = ",es_nmax   
  call message(AppMsg,Verbose)


! Read the pattern library
  call CAL_getEventPatterns(patterns, eduThreshold)

  call pat_init(patterns,edupat)  ! Initialize EDUSOFT

  eduThreshold = 10
    
  do ii = 0, 1023 
    offX( ii ) = 0
    offY( ii ) = 0
  enddo
  do jj = 1, dy
   do ii = 1, dx
    im( ii, jj ) = 0
   enddo
  enddo  

  im( 20, 10 ) = 100

  call edusoft(edumode,edupat,dx,dy,im,eduThreshold,  &
                   offX(0:x0+dx-1),offY(0:y0+dy-1),x0,y0, &
                   es_nmax,eduout)
 
  write(errstr,*) "EDUSOFT OUTPUT :"
  call message(AppMsg,Verbose)
  write(errstr,*) "eduout%nevent = ",eduout%nevent
  call message(AppMsg,Verbose)
  if( eduout%nevent > 0 ) then
   write(errstr,*) "eduout%evt(1)%x = ",eduout%evt(1)%x   
   call message(AppMsg,Verbose)
   write(errstr,*) "eduout%evt(1)%y = ",eduout%evt(1)%y   
   call message(AppMsg,Verbose)
   write(errstr,*) "eduout%evt(1)%y = ",eduout%evt(1)%e1
   call message(AppMsg,Verbose)
   write(errstr,*) "eduout%evt(1)%y = ",eduout%evt(1)%e2
   call message(AppMsg,Verbose)
  endif

! CALL projectEventsCounts

 do npattern = 0,edu_npat-1
    do jr = -1,1
       patabove(:,jr,npattern) = patterns(npattern,jr+2,1:3)
    enddo
 enddo
 where (patabove > 1)
       patabove = 0
 endwhere
 patabove(0,0,:) = 0  

! evt        : Array of event structure
! patabove   : Geometry of event outside central pixel
! image      : Image upon which to project

 evt(1)%rawx = 5
 evt(1)%rawy = 5
 evt(1)%pattern = 1
 evt(1)%e1   = 1000
 evt(1)%e2   = 50
 
 call message(AppMsg,Verbose," call  projectEventsCounts()")
 call message(AppMsg,Verbose)
 call projectEventsCounts(evt, patabove, imagein)

! CALL projectEventsEnergy

 call message(AppMsg,Verbose," call  projectEventsEnergy()")
 call projectEventsEnergy(evt, patabove, imagein)

! CALL  addFilename 

 ev_set = dataSet(filename, Modify)
 ev_tab = table(ev_set, 0)  ! Extension name depends on mode

 call message(AppMsg,Verbose," CALL addFilename")
 call addFilename(block(ev_tab), filename,"zozo")

! CALL  changeCase 

 write(errstr,*) changeCase("call CHANGEcASE",0)
 call message(AppMsg,Verbose)

! CALL  getCcd 

 write(errstr,*) "CCD NUMBER =", getCcd(block(ev_tab))
 call message(AppMsg,Verbose)

! CALL  getMode 

 call message(AppMsg,Verbose," CALL getMode") 
 call getMode(ev_tab, imag, timi, redI, comp)
 write(errstr,*) "imagin=", imag," timing=",timi," redImaging=", &
   redI, " compTiming=",comp
 call message(AppMsg,Verbose)

! CALL  keywordDone 

 call message(AppMsg,Verbose," CALL keywordDone")  
 call keywordDone(block(ev_tab), taskname, "ZOZO")

 write(errstr,*) "Was it done ? :", wasDone(block(ev_tab), "ZOZO" )
 call message(AppMsg,Verbose)

! CALL  keywordRemove

 call message(AppMsg,Verbose," CALL keywordRemove")  
 call keywordRemove(block(ev_tab), "ZOZO" )

! CALL  wasDone 

 call message(AppMsg,Verbose," CALL keywordDone")  
 write(errstr,*) "Was it done ? :",wasDone(block(ev_tab), "ZOZO" )
 call message(AppMsg,Verbose)

! CALL  equalKeywords 

 call message(AppMsg,Verbose," CALL equalKeywords")  
 write(errstr,*) "Compat. ? :", equalKeywords(attributable(ev_tab), &
 attributable(ev_set), keycompat,.false.)
 call message(AppMsg,Verbose)

! These calls need a parameter list, which we haven't here
! See evlistcomb_mod.f90 for examples of how to call.
! CALL  sizeListParam 
! CALL  readListParam 


END PROGRAM check
