//
//	check correctness of EnergyCombinator class
//
#include <cassert>
#include <iostream>

#include "EnergyCombinator.h"
#include "CalDataVector.h"


#include "common.h"

int main()
{
	float e1[]		= { 1000., 2000., 3000. };
        int16 e2[]		= {  100,  200,  300 },
	      e3[]		= {   70,   60,   50 },
	      e4[]		= {  140,  110,  160 };
	int8  pattern[] =  { 2,  6, 9 };
        float locbkg[]  = {10.,20.,30.};
        int8  peripix[] =  { 2,  5, 0};
	int32  flag[]    =  { 0,  0, 0}; // 128, 256

	unsigned N = NUM_OF(e1);
	CalInt16Vector e2v(e2, N),
		       e3v(e3, N),
		       e4v(e4, N);

	CalReal32Vector e1v(e1, N),
                        locbkgV(locbkg, N);

	CalReal32Vector  pha(0, N); //  0 signifies that CalVector class should
				    //	allocate memory
	CalInt8Vector  patternV(pattern, N);
	CalInt8Vector  peripixV(peripix, N);
	CalInt32Vector flagV(flag, N);

// Setting of coefficients (the dimension of arrays is the number of patterns).


        PatternLibServer *pattLib = calServer.getAtom(pattLib);
        unsigned nbpat =  pattLib->patterns() ;
        cout << "Number of patterns: " << nbpat << endl;

	vector<float> e1Coef(nbpat),
				  e2Coef(nbpat),
				  e3Coef(nbpat),
				  e4Coef(nbpat);

        for (unsigned n=0; n<4; ++n) { // Mono and bi pixels.
	e1Coef[n] = 1.f; e2Coef[n] = 1.f; e3Coef[n] = 0.f;  e4Coef[n] = -1.f;
        }
        for (unsigned n=5; n<nbpat; ++n) { // others
	e1Coef[n] = 1.f; e2Coef[n] = 1.f; e3Coef[n] = 1.f;  e4Coef[n] = 1.f;
        }


	// instantiate EnergyCombinator object
	EnergyCombinator ec(e1Coef, e2Coef, e3Coef, e4Coef);

      cout << "locbkgV " <<locbkgV[0]<<"|"<<locbkgV[1]<< "|"<<locbkgV[2]<<endl;





        // without any optional argument (1st order)
	ec.combine(e1v, e2v, patternV, pha );

     	cout << "Number of events: " << N << endl;
  	float sum = 0.;
	for (unsigned n=0; n<N; ++n) {
		cout<<e1v[n]<<'/'<<e2v[n]<<'/'<<e3v[n]<<'/'<<e4v[n]<<" => "
			<<pha[n]<<endl;
		sum += pha[n];
	}

	cout << "sum " << sum  << endl;    
	assert(sum == 6600 );


        // Without optional arguments e3v, e4v, peripixV, flagV
     	ec.combine(e1v, e2v, patternV, pha, locbkgV );

     	cout << "Number of events: " << N << endl;
      cout << "locbkgV " <<locbkgV[0]<<"|"<<locbkgV[1]<< "|"<<locbkgV[2]<<endl;
        sum = 0.;
	for (unsigned n=0; n<N; ++n) {
		cout<<e1v[n]<<'/'<<e2v[n]<<'/'<<e3v[n]<<'/'<<e4v[n]<<" => "
			<<pha[n]<<endl;
		sum += pha[n];
	}

	cout << "sum " << sum  << endl;    
	assert(sum == 6400 );


        // without optional argument locbkgV
       	ec.combine(e1v, e2v, patternV, pha, e3v, e4v, peripixV, flagV );

     	cout << "Number of events: " << N << endl;
        sum = 0.;
	for (unsigned n=0; n<N; ++n) {
		cout<<e1v[n]<<'/'<<e2v[n]<<'/'<<e3v[n]<<'/'<<e4v[n]<<" => "
			<<pha[n]<<endl;
		sum += pha[n];
	}

	cout << "sum " << sum  << endl;    
	assert(sum == 6960);



	//	do the combination
       ec.combine(e1v, e2v, patternV, pha, locbkgV, e3v, e4v, peripixV, flagV);

        cout << "Number of events: " << N << endl;
      cout << "locbkgV " <<locbkgV[0]<<"|"<<locbkgV[1]<< "|"<<locbkgV[2]<<endl;
	sum = 0.;
	for (unsigned n=0; n<N; ++n) {
		cout<<e1v[n]<<'/'<<e2v[n]<<'/'<<e3v[n]<<'/'<<e4v[n]<<" => "
			<<pha[n]<<endl;
		sum += pha[n];
	}
	
	cout << "sum " << sum  << endl;
	assert(sum == 5810);



	return 0;
}
